@@ -4,16 +4,17 @@
 #  (C) Tom Molesworth 2011,
 #      Paul Evans, 2011-2015 -- leonerd@leonerd.org.uk
 
-package Tickit::Widget::Tabbed;
+use 5.026;
+use Object::Pad 0.22;
+
+class Tickit::Widget::Tabbed 0.022
+        extends Tickit::ContainerWidget;
+
+Tickit::Widget->VERSION("0.12");
+Tickit::Window->VERSION("0.23");
 
-use strict;
-use warnings;
-use parent qw(Tickit::ContainerWidget);
-BEGIN {
-        Tickit::Widget->VERSION("0.12");
-        Tickit::Window->VERSION("0.23");
-}
 use Tickit::Style;
+use constant KEYPRESSES_FROM_STYLE => 1;
 
 use Carp;
 use Tickit::Pen;
@@ -21,8 +22,6 @@ use List::Util qw(max);
 
 use Tickit::Widget::Tabbed::Ribbon;
 
-our $VERSION = '0.020';
-
 =head1 NAME
 
 Tickit::Widget::Tabbed - provide tabbed window support
@@ -84,7 +83,12 @@ style_definition base =>
         active_fg => 14,
         more_fg => "cyan",
         more_left => "<..",
-        more_right => "..>";
+        more_right => "..>",
+
+        '<C-n>'        => "next_tab",
+        '<C-p>'        => "prev_tab",
+        '<C-PageDown>' => "next_tab",
+        '<C-PageUp>'   => "prev_tab";
 
 use constant WIDGET_PEN_FROM_STYLE => 1;
 
@@ -92,9 +96,6 @@ use constant WIDGET_PEN_FROM_STYLE => 1;
 
 =cut
 
-sub TAB_CLASS { shift->{tab_class} || "Tickit::Widget::Tabbed::Tab" }
-sub RIBBON_CLASS { shift->{ribbon_class} || "Tickit::Widget::Tabbed::Ribbon" }
-
 =head2 new
 
 Instantiate a new tabbed window.
@@ -109,82 +110,74 @@ Takes the following named parameters:
 
 =cut
 
-sub new {
-        my $class = shift;
-        my %args = @_;
-        my $self = $class->SUPER::new(%args);
+has $_tab_class;
+method TAB_CLASS { $_tab_class || "Tickit::Widget::Tabbed::Tab" }
 
-        $self->{tab_class}  = delete($args{tab_class});
-        $self->{ribbon_class} = delete($args{ribbon_class});
+has $_ribbon_class;
+method RIBBON_CLASS { $_ribbon_class || "Tickit::Widget::Tabbed::Ribbon" }
+
+has $_ribbon;
+method ribbon { $_ribbon }
+
+has @_child_window_geometry;
+
+method BUILD ( %args ) {
+        $_tab_class    = delete($args{tab_class});
+        $_ribbon_class = delete($args{ribbon_class});
 
         $self->tab_position(delete($args{tab_position}) || 'top');
+        # sets $_ribbon
 
-        my $ribbon = $self->{ribbon};
-
-        $ribbon->set_style( $self->get_style_pen("ribbon")->getattrs );
-
-        return $self;
+        $_ribbon->set_style( $self->get_style_pen("ribbon")->getattrs );
 }
 
 # Positions for the four screen edges - these will return appropriate sizes
 # for the tab and child subwindows
-sub _window_position_left {
-        my $self = shift;
-        my $ribbon = $self->{ribbon};
-        my $label_width = $ribbon->cols;
+method _window_position_left () {
+        my $label_width = $_ribbon->cols;
         return 0, 0, $self->window->lines, $label_width,
                0, $label_width, $self->window->lines, $self->window->cols - $label_width;
 }
 
-sub _window_position_right {
-        my $self = shift;
-        my $ribbon = $self->{ribbon};
-        my $label_width = $ribbon->cols;
+method _window_position_right () {
+        my $label_width = $_ribbon->cols;
         return 0, $self->window->cols - $label_width, $self->window->lines, $label_width,
                0, 0, $self->window->lines, $self->window->cols - $label_width;
 }
 
-sub _window_position_top {
-        my $self = shift;
-        my $ribbon = $self->{ribbon};
-        my $label_height = $ribbon->lines;
+method _window_position_top () {
+        my $label_height = $_ribbon->lines;
         $label_height = 1 unless $self->window->lines > $label_height;
         return 0, 0, $label_height, $self->window->cols,
                $label_height, 0, max(1, $self->window->lines - $label_height), $self->window->cols;
 }
 
-sub _window_position_bottom {
-        my $self = shift;
-        my $ribbon = $self->{ribbon};
-        my $label_height = $ribbon->lines;
+method _window_position_bottom () {
+        my $label_height = $_ribbon->lines;
         $label_height = 1 unless $self->window->lines > $label_height;
         return $self->window->lines - $label_height, 0, $label_height, $self->window->cols,
                0, 0, max(1, $self->window->lines - $label_height), $self->window->cols;
 }
 
-sub on_style_changed_values {
-        my $self = shift;
-        my %values = @_;
-
+method on_style_changed_values (%values) {
         if( grep { $_ =~ m/^ribbon_/ } keys %values ) {
-                $self->{ribbon}->set_style( $self->get_style_pen("ribbon")->getattrs );
+                $_ribbon->set_style( $self->get_style_pen("ribbon")->getattrs );
         }
 }
 
-sub reshape {
-        my $self = shift;
+method reshape () {
         my $window = $self->window or return;
         my $tab_position = $self->tab_position;
         my @positions = $self->${\"_window_position_$tab_position"}();
-        if( my $ribbon_window = $self->{ribbon}->window ) {
+        if( my $ribbon_window = $_ribbon->window ) {
                 $ribbon_window->change_geometry( @positions[0..3] );
         }
         else {
                 my $ribbon_window = $window->make_sub( @positions[0..3] );
-                $self->{ribbon}->set_window( $ribbon_window );
+                $_ribbon->set_window( $ribbon_window );
         }
-        $self->{child_window_geometry} = [ @positions[4..7] ];
-        foreach my $tab ( $self->{ribbon}->tabs ) {
+        @_child_window_geometry = @positions[4..7];
+        foreach my $tab ( $_ribbon->tabs ) {
                 my $child = $tab->widget;
                 if( my $child_window = $child->window ) {
                         $child_window->change_geometry( @positions[4..7] );
@@ -196,80 +189,58 @@ sub reshape {
         }
 }
 
-sub _max_child_lines
-{
-        my $self = shift;
-        my $ribbon = $self->{ribbon};
-        return max( 1, map { $_->widget->requested_lines } $ribbon->tabs );
+method _max_child_lines () {
+        return max( 1, map { $_->widget->requested_lines } $_ribbon->tabs );
 }
 
-sub _max_child_cols
-{
-        my $self = shift;
-        my $ribbon = $self->{ribbon};
-        return max( 1, map { $_->widget->requested_cols } $ribbon->tabs );
+method _max_child_cols () {
+        return max( 1, map { $_->widget->requested_cols } $_ribbon->tabs );
 }
 
-sub lines
-{
-        my $self = shift;
-        my $ribbon = $self->{ribbon};
-
-        if( $ribbon->orientation eq "horizontal" ) {
-                return $ribbon->lines + $self->_max_child_lines;
+method lines () {
+        if( $_ribbon->orientation eq "horizontal" ) {
+                return $_ribbon->lines + $self->_max_child_lines;
         }
         else {
-                return max( $ribbon->lines, $self->_max_child_lines );
+                return max( $_ribbon->lines, $self->_max_child_lines );
         }
 }
 
-sub cols
-{
-        my $self = shift;
-        my $ribbon = $self->{ribbon};
-
-        if( $ribbon->orientation eq "horizontal" ) {
-                return max( $ribbon->cols, $self->_max_child_cols );
+method cols () {
+        if( $_ribbon->orientation eq "horizontal" ) {
+                return max( $_ribbon->cols, $self->_max_child_cols );
         }
         else {
-                return $ribbon->cols + $self->_max_child_cols;
+                return $_ribbon->cols + $self->_max_child_cols;
         }
 }
 
 # All the child widgets
-sub children {
-        my $self = shift;
-        my $ribbon = $self->{ribbon};
-        return $ribbon, map { $_->widget } $ribbon->tabs;
+method children () {
+        return $_ribbon, map { $_->widget } $_ribbon->tabs;
 }
 
 # The only focusable child widget is the active one
-sub children_for_focus {
-        my $self = shift;
+method children_for_focus () {
         return $self->active_tab_widget;
 }
 
-sub _new_child_window
-{
-        my $self = shift;
-        my ( $visible ) = @_;
-
+method _new_child_window ( $visible ) {
         my $window = $self->window or return undef;
 
-        my $child_window = $window->make_hidden_sub( @{ $self->{child_window_geometry} } );
+        my $child_window = $window->make_hidden_sub( @_child_window_geometry );
         $child_window->show if $visible;
 
         return $child_window;
 }
 
-sub window_lost {
-        my $self = shift;
-        $self->SUPER::window_lost(@_);
-        $_->widget->set_window(undef) for $self->{ribbon}->tabs;
+method window_lost ( $win ) {
+        $self->SUPER::window_lost( $win );
+        $_->widget->set_window(undef) for $_ribbon->tabs;
 
-        undef $self->{child_window_geometry};
+        undef @_child_window_geometry;
 
-        $self->{ribbon}->set_window(undef);
+        $_ribbon->set_window(undef);
 }
 
 =head2 tab_position
@@ -278,46 +249,44 @@ Accessor for the tab position (top, left, right, bottom).
 
 =cut
 
-sub tab_position {
-        my $self = shift;
-        if(@_) {
-                my $pos = shift;
-                my $orientation = ( $pos eq "top" or $pos eq "bottom" ) ? "horizontal" :
-                                  ( $pos eq "left" or $pos eq "right" ) ? "vertical" :
-                                  croak "Unrecognised value for ->tab_position: $pos";
+has $_tab_position;
+method tab_position ( $pos = return $_tab_position ) {
+        my $orientation = ( $pos eq "top" or $pos eq "bottom" ) ? "horizontal" :
+                          ( $pos eq "left" or $pos eq "right" ) ? "vertical" :
+                          croak "Unrecognised value for ->tab_position: $pos";
 
-                if( !$self->{ribbon} or $self->{ribbon}->orientation ne $orientation ) {
-                        my %args = (
-                                tabbed => $self,
-                                tab_position => $pos,
-                        );
-                        if( my $old_ribbon = $self->{ribbon} ) {
-                                $old_ribbon->set_window( undef );
-                                $args{tabs} = [ $old_ribbon->tabs ];
-                                $args{active_tab_index} = $old_ribbon->active_tab_index;
-                                $args{pen}  = $old_ribbon->pen;
-                                $args{active_pen} = $old_ribbon->active_pen;
-                                undef $self->{ribbon};
-                        }
-                        $self->{ribbon} = $self->RIBBON_CLASS->new_for_orientation(
-                                $orientation, %args
-                        );
-                        $self->{ribbon}->set_style( $args{pen}->getattrs ) if $args{pen};
+        if( !$_ribbon or $_ribbon->orientation ne $orientation ) {
+                my %args = (
+                        tabbed => $self,
+                        tab_position => $pos,
+                );
+                if( my $old_ribbon = $_ribbon ) {
+                        $old_ribbon->window->close;
+                        $old_ribbon->set_window( undef );
+                        $args{tabs} = [ $old_ribbon->tabs ];
+                        $args{active_tab_index} = $old_ribbon->active_tab_index;
+                        $args{pen}  = $old_ribbon->pen;
+                        $args{active_pen} = $old_ribbon->active_pen;
+                        undef $_ribbon;
                 }
-
-                $self->{tab_position} = $pos;
-                undef $self->{child_window_geometry};
-
-                $self->reshape if $self->window;
-                $self->redraw;
+                $_ribbon = $self->RIBBON_CLASS->new_for_orientation(
+                        $orientation, %args
+                );
+                $_ribbon->set_style( $args{pen}->getattrs ) if $args{pen};
         }
-        return $self->{tab_position};
+
+        $_tab_position = $pos;
+        undef @_child_window_geometry;
+
+        $self->reshape if $self->window;
+        $self->redraw;
+
+        return $_tab_position;
 }
 
-sub _tabs_changed {
-        my $self = shift;
+method _tabs_changed () {
         $self->reshape if $self->window;
-        $self->{ribbon}->redraw if $self->{ribbon}->window;
+        $_ribbon->redraw if $_ribbon->window;
 }
 
 =head2 active_tab_index
@@ -326,7 +295,7 @@ Returns the 0-based index of the currently-active tab.
 
 =cut
 
-sub active_tab_index { shift->{ribbon}->active_tab_index }
+method active_tab_index () { $_ribbon->active_tab_index }
 
 =head2 active_tab
 
@@ -334,7 +303,7 @@ Returns the currently-active tab as a tab object. See below.
 
 =cut
 
-sub active_tab { shift->{ribbon}->active_tab }
+method active_tab () { $_ribbon->active_tab }
 
 =head2 active_tab_widget
 
@@ -344,8 +313,8 @@ Returns the widget in the currently active tab.
 
 # Old name
 *tab = \&active_tab_widget;
-sub active_tab_widget {
-        my $self = shift; $self->active_tab && $self->active_tab->widget
+method active_tab_widget () {
+        $self->active_tab && $self->active_tab->widget
 }
 
 =head2 add_tab
@@ -365,15 +334,10 @@ Remaining form a hash:
 
 =cut
 
-sub add_tab {
-        my $self = shift;
-        my ($child, %opts) = @_;
-
-        my $ribbon = $self->{ribbon};
-
+method add_tab ( $child, %opts ) {
         my $tab = $self->TAB_CLASS->new( $self, widget => $child, %opts );
 
-        $ribbon->append_tab( $tab );
+        $_ribbon->append_tab( $tab );
 
         return $tab;
 }
@@ -384,7 +348,7 @@ Remove tab given by 0-based index or tab object.
 
 =cut
 
-sub remove_tab { shift->{ribbon}->remove_tab( @_ ) }
+method remove_tab { $_ribbon->remove_tab( @_ ) }
 
 =head2 move_tab
 
@@ -393,7 +357,7 @@ positions.
 
 =cut
 
-sub move_tab { shift->{ribbon}->move_tab( @_ ) }
+method move_tab { $_ribbon->move_tab( @_ ) }
 
 =head2 activate_tab
 
@@ -401,65 +365,52 @@ Switch to the given tab; by 0-based index, or object.
 
 =cut
 
-sub activate_tab { shift->{ribbon}->activate_tab( @_ ) }
+method activate_tab { $_ribbon->activate_tab( @_ ) }
 
 =head2 next_tab
 
-Switch to the next tab.
+Switch to the next tab. This may be bound as a key action.
 
 =cut
 
-sub next_tab { shift->{ribbon}->next_tab }
+*key_next_tab = \&next_tab;
+method next_tab (@) { $_ribbon->next_tab }
 
 =head2 prev_tab
 
-Switch to the previous tab.
+Switch to the previous tab. This may be bound as a key action.
 
 =cut
 
-sub prev_tab { shift->{ribbon}->prev_tab }
+*key_prev_tab = \&prev_tab;
+method prev_tab (@) { $_ribbon->prev_tab }
 
-sub child_resized {
-        my $self = shift;
+method child_resized () {
         $self->reshape;
 }
 
-sub on_key {
-        my $self = shift;
-        my ($ev) = @_;
-
-        return 1 if $self->{ribbon}->on_key(@_);
+method on_key ( $ev ) {
+        return 1 if $_ribbon->on_key( $ev );
 
         return 0 unless $ev->type eq "key";
 
         my $str = $ev->str;
-        if($str eq 'C-PageUp') {
-                $self->prev_tab;
-                return 1;
-        }
-        if($str eq 'C-PageDown') {
-                $self->next_tab;
-                return 1;
-        }
         if($str =~ m/^M-(\d)$/ ) {
                 my $index = $1 - 1;
-                $self->activate_tab( $index ) if $index < $self->{ribbon}->tabs;
+                $self->activate_tab( $index ) if $index < $_ribbon->tabs;
                 return 1;
         }
         return 0;
 }
 
-sub render_to_rb {
-        my $self = shift;
-        my ( $rb, $rect ) = @_;
-
+method render_to_rb ( $rb, $rect ) {
         # Just clear the child area if we have nothing better to do
         $rb->eraserect( $rect );
 }
 
-package Tickit::Widget::Tabbed::Tab;
+class # hide from indexer
+        Tickit::Widget::Tabbed::Tab :repr(HASH);
 
-use 5.010; # for //= operator
 use Scalar::Util qw( weaken );
 use Tickit::Utils qw( textwidth );
 
@@ -470,17 +421,20 @@ C<active_tab>.
 
 =cut
 
-sub new {
-        my $class = shift;
-        my ( $tabbed, %args ) = @_;
-        my $self = bless {
-                tabbed => $tabbed,
-                widget => $args{widget},
-                label  => $args{label},
-                active => 0,
-        }, $class;
-        weaken( $self->{tabbed} );
-        return $self;
+has $_tabbed;
+
+has $_widget;
+has $_label;
+has $_active = 0;
+
+has $_on_activated;
+has $_on_deactivated;
+
+method BUILD ( $tabbed, %args ) {
+        weaken( $_tabbed = $tabbed );
+
+        $_widget = $args{widget};
+        $_label  = $args{label};
 }
 
 =head2 index
@@ -489,9 +443,8 @@ Returns the 0-based index of this tab
 
 =cut
 
-sub index {
-        my $self = shift;
-        return $self->{tabbed}->{ribbon}->_tab2index( $self );
+method index () {
+        return $_tabbed->ribbon->_tab2index( $self );
 }
 
 =head2 widget
@@ -500,7 +453,7 @@ Returns the C<Tickit::Widget> contained by this tab
 
 =cut
 
-sub widget { shift->{widget} }
+method widget () { $_widget }
 
 =head2 label
 
@@ -508,12 +461,13 @@ Returns the current label text
 
 =cut
 
-sub label_width {
-        my $self = shift;
-        return $self->{label_width} //= textwidth( $self->{label} );
+has $_label_width;
+
+method label_width () {
+        return $_label_width //= textwidth( $_label );
 }
 
-sub label { shift->{label} }
+method label () { $_label }
 
 =head2 set_label
 
@@ -521,11 +475,10 @@ Set new label text for the tab
 
 =cut
 
-sub set_label {
-        my $self = shift;
-        ( $self->{label} ) = @_;
-        undef $self->{label_width};
-        $self->{tabbed}->_tabs_changed if $self->{tabbed};
+method set_label ( $label ) {
+        $_label = $label;
+        undef $_label_width;
+        $_tabbed->_tabs_changed if $_tabbed;
 }
 
 =head2 is_active
@@ -534,9 +487,8 @@ Returns true if this tab is the currently active one
 
 =cut
 
-sub is_active {
-        my $self = shift;
-        return $self->{tabbed}->active_tab == $self;
+method is_active () {
+        return $_tabbed->active_tab == $self;
 }
 
 =head2 activate
@@ -545,20 +497,17 @@ Activate this tab
 
 =cut
 
-sub activate {
-        my $self = shift;
-        $self->{tabbed}->activate_tab( $self );
+method activate () {
+        $_tabbed->activate_tab( $self );
 }
 
-sub _activate {
-        my $self = shift;
+method _activate () {
         $self->widget->window->show if $self->widget->window;
-        $self->${\$self->{on_activated}}() if $self->{on_activated};
+        $self->$_on_activated() if $_on_activated;
 }
 
-sub _deactivate {
-        my $self = shift;
-        $self->${\$self->{on_deactivated}}() if $self->{on_deactivated};
+method _deactivate () {
+        $self->$_on_deactivated() if $_on_deactivated;
         $self->widget->window->hide if $self->widget->window;
 }
 
@@ -568,11 +517,7 @@ Set a callback or method name to invoke when the tab is activated
 
 =cut
 
-sub set_on_activated
-{
-        my $self = shift;
-        ( $self->{on_activated} ) = @_;
-}
+method set_on_activated ($) { $_on_activated = $_[0]; }
 
 =head2 set_on_deactivated
 
@@ -580,11 +525,7 @@ Set a callback or method name to invoke when the tab is deactivated
 
 =cut
 
-sub set_on_deactivated
-{
-        my $self = shift;
-        ( $self->{on_deactivated} ) = @_;
-}
+method set_on_deactivated ($) { $_on_deactivated = $_[0]; }
 
 =head2 pen
 
@@ -595,25 +536,20 @@ pen on the tab, use the C<set_pen> method instead.
 
 =cut
 
-sub _has_pen { defined shift->{pen} }
+has $_pen;
 
-sub pen {
-        my $self = shift;
-        return $self->{pen};
+method _has_pen () { defined $_pen }
+
+method pen () { $_pen }
+
+method set_pen ($) {
+        $_pen = $_[0];
+        $_tabbed->_tabs_changed if $_tabbed;
 }
 
-sub set_pen {
-        my $self = shift;
-        ( $self->{pen} ) = @_;
-        $self->{tabbed}->_tabs_changed if $self->{tabbed};
-}
-
-sub on_mouse {
-        my $self = shift;
-        my ( $type, $button, $line, $col ) = @_;
-
+method on_mouse ( $type, $button, $line, $col ) {
         return 0 unless $type eq "press" && $button == 1;
-        $self->{tabbed}->activate_tab( $self );
+        $_tabbed->activate_tab( $self );
         return 1;
 }
 
